#include "stdafx.h"

const int ONEWEEK = 1000*60*60*24*7;

CDigitalTimeCourse :: CDigitalTimeCourse() { 
	m_aiTimes=NULL;
	m_iSegCount=0; 
	}
	
CDigitalTimeCourse :: ~CDigitalTimeCourse() { 
	if( m_aiTimes!=NULL ) 
		delete [] m_aiTimes;
	}

bool CDigitalTimeCourse :: Parse( const char * pch ) {
	if( m_aiTimes!=NULL ) 
		delete [] m_aiTimes;
	m_iTotalTime=0;

	//-- make a copy of buffer
	int iLen = strlen(pch);
	char * achTemp = new char[iLen+1];
	strcpy( achTemp, pch );
	while( iLen>0 && achTemp[iLen-1]==' ' )
		achTemp[--iLen]=0;

	//-- if empty, no segments
	if( iLen==0 ) {
		m_aiTimes=NULL;
		m_iSegCount=0;
		return true;
		}
	
	//-- count commas 
	int i, iPos=0, iCommas=0; 
	for( i=0; i<iLen; ++i ) 
		if( pch[i]==',' )
			++iCommas;
			
	//-- allocate space for time course
	m_aiTimes=new int[m_iSegCount=iCommas+1];
	if( m_aiTimes==NULL ) 
		return false;

	//-- convert each ,..., to an int value		
	iCommas=0;
	for( i=0; i<=iLen; ++i ) 
		if( achTemp[i]==',' || i==iLen ) {
			achTemp[i]=0;
			m_aiTimes[iCommas]=atoi( achTemp+iPos );
			m_iTotalTime += m_aiTimes[iCommas];
			iCommas++;
			iPos=i+1;
			}
	//-- 
	delete [] achTemp;
	return true;
	} 

void CDigitalTimeCourse :: SetPos0( ) { 
	m_iCurrSeg=0;
	m_iCurrSegOffset=0;
	m_iNextChange = ( m_iSegCount>0 ? m_aiTimes[0] : ONEWEEK );
	}

//----------------------------	
CDigitalProtocol :: CDigitalProtocol( ){
	m_iDuration=10000;
	m_iIterations=1;
	m_iLines=8;
	m_acTimeString[0].Set("1000,2000");
	m_acTimeString[1].Set("2000,2000");
	for( int i=2; i<MAXDOLINES; ++i ) 
		m_acTimeString[i].Set("");
	}

CDigitalProtocol :: ~CDigitalProtocol( ){
	}

bool CDigitalProtocol :: LoadFile( const char * pchFile ){
	int iVer;
	//-- Open File
	FILE * h = fopen( pchFile, "rb" );
	if( h == NULL ) 
		return false;
	//-- Read Header
	if( fread( &iVer, sizeof(iVer), 1, h )!=1 
		|| fread( &m_iDuration, sizeof(m_iDuration), 1, h )!=1 
		|| fread( &m_iIterations, sizeof(m_iIterations), 1, h )!=1 
		|| fread( &m_iLines, sizeof(m_iLines), 1, h )!=1 ) {
		fclose(h);
		return false;
		}

	//-- Read Time Course Data 
	for( int iTC=0; iTC<MAXDOLINES; ++iTC )
		if( ! FileLoadString( h, &(m_acTimeString[iTC]) ) ) {
			fclose(h);
			return false;
			}

	fclose(h);
	return true;
	}

bool CDigitalProtocol :: SaveFile( const char * pchFile ){
	//-- Open File
	FILE * h = fopen( pchFile, "wb" );
	if( h == NULL ) 
		return false;
		
	//-- Write Header
	int iVer=1;
	if ( fwrite( &iVer, sizeof(iVer), 1, h )!=1
		|| fwrite( &m_iDuration, sizeof(m_iDuration), 1, h )!=1 
		|| fwrite( &m_iIterations, sizeof(m_iIterations), 1, h )!=1 
		|| fwrite( &m_iLines, sizeof(m_iLines), 1, h )!=1 ) { 
		fclose(h);
		return false;
		}
		
	//-- Write Time Course Strings
	for( int iTC=0; iTC<MAXDOLINES; ++iTC )
		if( ! FileSaveString( h, m_acTimeString[iTC].Buffer() ) ) {
			fclose(h);
			return false;
			}
			
	fclose(h);
	return true;
	}

bool CDigitalProtocol :: GenPrepare( int iGenRate ) { 
	m_iGenIteration=0;
	m_iGenSample=0;
	m_iGenRate=iGenRate;
	for( int iTC=0; iTC<MAXDOLINES; ++iTC ) {
		m_aTimeCourse[iTC].Parse( m_acTimeString[iTC].Buffer() );
		m_aTimeCourse[iTC].SetPos0();
		}
		
	return true;
	}

int CDigitalProtocol :: NextChange( ) { 
	if( m_iLines==0 ) 
		return 0;
	int iNextChange = m_aTimeCourse[0].m_iNextChange;
	for( int iLine=1; iLine<MAXDOLINES && iLine<m_iLines; ++iLine )
		iNextChange = min( iNextChange, m_aTimeCourse[iLine].m_iNextChange );
	return MulDiv( iNextChange, m_iGenRate, 1000 );	// return value converted to sample count
	}

unsigned int CDigitalProtocol :: Sample( ) { 
	unsigned int iSample=0;
	for( int iLine=0; iLine<MAXDOLINES && iLine<m_iLines; ++iLine ) 
		if( m_aTimeCourse[iLine].m_iCurrSeg%2==1 ) 
			iSample |= (1<<iLine);
	return iSample;
	}

/* note : since SetPos returns boolean of whether any changed, could use : 
	while( SetPos( iPos ) ) ;
to jump to arbitrary position.
*/
bool CDigitalProtocol :: SetPos( int iPos ) { 
	bool bChanged=false;
	for( int iLine=0; iLine<m_iLines && iLine<MAXDOLINES; ++iLine ) { 
		CDigitalTimeCourse * pTC = &(m_aTimeCourse[iLine]);
		
		//-- if past nextchange pt., advance to next segment
		if( pTC->m_iNextChange <= iPos ) { 
			if( pTC->m_iCurrSeg+1==pTC->m_iSegCount )			// if no more segs, maintain last segment indefinitely
				pTC->m_iNextChange=ONEWEEK;
			else {												// else go to next seg
				pTC->m_iCurrSegOffset += pTC->m_aiTimes[pTC->m_iCurrSeg];
				++pTC->m_iCurrSeg;
				pTC->m_iNextChange += pTC->m_aiTimes[pTC->m_iCurrSeg];
				bChanged=true;
				}
			}
		}
	return bChanged;
	}

int CDigitalProtocol :: SampleCount( int iGenRate ) {  
	return MulDiv( m_iDuration, iGenRate, 1000 );
	}
	
bool CDigitalProtocol :: GenerateData( unsigned int * pData, int iCount, bool bRepeat ) { 
	int iPos;							// position in millisec
	int iNextChange = NextChange();		// min nextchange sample pos
	unsigned int iSample = Sample();	// value to output 
	int iDurationSamples = MulDiv( m_iDuration, m_iGenRate, 1000 );

	for( int i=0; i<iCount; ++i, ++m_iGenSample ) { 
		//-- start next iteration if necc.
		if( m_iGenSample >= iDurationSamples ) {
			if( ++m_iGenIteration<m_iIterations || bRepeat ) {
				for( int iTC=0; iTC<MAXDOLINES; ++iTC )
					m_aTimeCourse[iTC].SetPos0();
				iNextChange=NextChange();
				m_iGenSample=0;
				}
			else { 
				iSample=0;
				iNextChange=ONEWEEK;
				}
			}
		//-- advance to next segment if up to next change point		
		if( m_iGenSample >= iNextChange ) {		
			iPos = MulDiv( m_iGenSample, 1000, m_iGenRate );
			SetPos( iPos );
			iNextChange = NextChange();
			iSample = Sample();
			}
		//-- fill in a value
		pData[i]=iSample;
		}
		
	return true;
	}
	
void CDigitalProtocol :: Inject( int iTC, int iTime ) { 
	bool bSet=false;
	CDigitalTimeCourse o;
	o.Parse( m_acTimeString[iTC].ConstBuffer() );
	
	char * buf = new char[m_acTimeString[iTC].Length()+100];
	buf[0]=0;
	
	int iElapsed=0;
	for( int iSeg=0; iSeg<o.m_iSegCount; ++iSeg ) {
		if( iElapsed + o.m_aiTimes[iSeg] == iTime )
			return; 
		if( ! bSet && iElapsed + o.m_aiTimes[iSeg] > iTime ) {
			sprintf( buf+strlen(buf), "%d,", iTime-iElapsed );
			o.m_aiTimes[iSeg] -=  ( iTime-iElapsed ); 
			iElapsed = iTime;
			bSet=true;
			}
		sprintf( buf+strlen(buf), iSeg==o.m_iSegCount-1?"%d":"%d,", o.m_aiTimes[iSeg] );
		iElapsed+=o.m_aiTimes[iSeg];
		}

	if( ! bSet && iTime>iElapsed )
		sprintf( buf+strlen(buf), o.m_iSegCount==0 ? "%d" : ",%d", iTime-iElapsed );
		
	m_acTimeString[iTC]=buf;
	delete [] buf;
	}

//----------------------------------------------------------
void ProtocolDigitalToAnalog( CDigitalProtocol * pDig, int iLine, double dLevel, CProtocol * pAna ) { 
	CDigitalTimeCourse * pDigTC = &(pDig->m_aTimeCourse[iLine]);
	pDigTC->Parse( pDig->m_acTimeString[iLine].ConstBuffer() );
	
	pAna->m_dYMin =-10.0;
	pAna->m_dYMax = 10.0;
	pAna->m_bRandom = true;
	pAna->m_iIterations = max(1,pDig->m_iIterations);

	char buf[16];
	for( int iSeg=0; iSeg<pDigTC->m_iSegCount; ++iSeg ) { 
		sprintf( buf, "%d", pDigTC->m_aiTimes[iSeg] );
		if( (iSeg%2)==0 ) 
			pAna->AddSegment( buf, dLevel, 0.0, false, false, 0.0, 0.0, "u", true );
		else 
			pAna->AddSegment( buf, 0.0, 0.0, false, false, 0.0, 0.0, "u", false );
		}
	}






